After selecting a task to kill, the oom killer iterates all processes and
kills all other user threads that share the same mm_struct in different
thread groups.

But in some extreme cases, the selected task happens to be a vfork child
of init process sharing the same mm_struct with it, which causes kernel
panic on init getting killed. This panic is observed in a busybox shell
that busybox itself is init, with a kthread keeps consuming memories.

Signed-off-by: Ming Liu <ming....@windriver.com>
---
 mm/oom_kill.c |   16 ++++++++--------
 1 files changed, 8 insertions(+), 8 deletions(-)

diff --git a/mm/oom_kill.c b/mm/oom_kill.c
index 314e9d2..7db4881 100644
--- a/mm/oom_kill.c
+++ b/mm/oom_kill.c
@@ -479,17 +479,17 @@ void oom_kill_process(struct task_struct *p, gfp_t 
gfp_mask, int order,
        task_unlock(victim);
 
        /*
         * Kill all user processes sharing victim->mm in other thread groups, if
         * any.  They don't get access to memory reserves, though, to avoid
         * depletion of all memory.  This prevents mm->mmap_sem livelock when an
         * oom killed thread cannot exit because it requires the semaphore and
         * its contended by another thread trying to allocate memory itself.
         * That thread will now get access to memory reserves since it has a
         * pending fatal signal.
         */
        for_each_process(p)
                if (p->mm == mm && !same_thread_group(p, victim) &&
-                   !(p->flags & PF_KTHREAD)) {
+                   !(p->flags & PF_KTHREAD) && !is_global_init(p)) {
                        if (p->signal->oom_score_adj == OOM_SCORE_ADJ_MIN)
                                continue;
 
@@ -479,17 +479,17 @@ void oom_kill_process(struct task_struct *p, gfp_t 
 
        /*
-        * Kill all user processes sharing victim->mm in other thread groups, if
-        * any.  They don't get access to memory reserves, though, to avoid
-        * depletion of all memory.  This prevents mm->mmap_sem livelock when an
-        * oom killed thread cannot exit because it requires the semaphore and
-        * its contended by another thread trying to allocate memory itself.
-        * That thread will now get access to memory reserves since it has a
-        * pending fatal signal.
+        * Kill all user processes except init sharing victim->mm in other
+        * thread groups, if any.  They don't get access to memory reserves,
+        * though, to avoid depletion of all memory.  This prevents mm->mmap_sem
+        * livelock when an oom killed thread cannot exit because it requires
+        * the semaphore and its contended by another thread trying to allocate
+        * memory itself. That thread will now get access to memory reserves
+        * since it has a pending fatal signal.
         */
        for_each_process(p)
-- 
1.7.0.4

--
To unsubscribe from this list: send the line "unsubscribe linux-kernel" in
the body of a message to majord...@vger.kernel.org
More majordomo info at  http://vger.kernel.org/majordomo-info.html
Please read the FAQ at  http://www.tux.org/lkml/

Reply via email to